@@ -12,7 +12,7 @@ module Net
 
       # This class encapsulates all operations done by clients on a user's
       # private keys. In practice, the client should never need a reference
-      # to a private key; instead, they grab a list of "identities" (public 
+      # to a private key; instead, they grab a list of "identities" (public
       # keys) that are available from the KeyManager, and then use
       # the KeyManager to do various private key operations using those
       # identities.
@@ -37,12 +37,13 @@ module Net
         attr_reader :options
 
         # Create a new KeyManager. By default, the manager will
-        # use the ssh-agent (if it is running).
+        # use the ssh-agent if it is running and the `:use_agent` option
+        # is not false.
         def initialize(logger, options={})
           self.logger = logger
           @key_files = []
           @key_data = []
-          @use_agent = true
+          @use_agent = !(options[:use_agent] == false)
           @known_identities = {}
           @agent = nil
           @options = options
@@ -97,7 +98,7 @@ module Net
         def each_identity
           prepared_identities = prepare_identities_from_files + prepare_identities_from_data
 
-          user_identities = load_identities(prepared_identities, false)
+          user_identities = load_identities(prepared_identities, false, true)
 
           if agent
             agent.identities.each do |key|
@@ -113,7 +114,7 @@ module Net
             end
           end
 
-          user_identities = load_identities(user_identities, true)
+          user_identities = load_identities(user_identities, !options[:non_interactive], false)
 
           user_identities.each do |identity|
             key = identity.delete(:public_key)
@@ -138,8 +139,8 @@ module Net
 
           if info[:key].nil? && info[:from] == :file
             begin
-              info[:key] = KeyFactory.load_private_key(info[:file], options[:passphrase], true)
-            rescue Exception, OpenSSL::OpenSSLError => e 
+              info[:key] = KeyFactory.load_private_key(info[:file], options[:passphrase], !options[:non_interactive])
+            rescue Exception, OpenSSL::OpenSSLError => e
               raise KeyManagerError, "the given identity is known, but the private key could not be loaded: #{e.class} (#{e.message})"
             end
           end
@@ -175,7 +176,7 @@ module Net
         # or if the agent is otherwise not available.
         def agent
           return unless use_agent?
-          @agent ||= Agent.connect(logger)
+          @agent ||= Agent.connect(logger, options[:agent_socket_factory])
         rescue AgentNotAvailable
           @use_agent = false
           nil
@@ -186,15 +187,24 @@ module Net
         # Prepares identities from user key_files for loading, preserving their order and sources.
         def prepare_identities_from_files
           key_files.map do |file|
-            public_key_file = file + ".pub"
-            if File.readable?(public_key_file)
-              { :load_from => :pubkey_file, :file => file }
-            elsif File.readable?(file)
-              { :load_from => :privkey_file, :file => file }
+            if readable_file?(file)
+              identity = {}
+              public_key_file = file + ".pub"
+              if readable_file?(public_key_file)
+                identity[:load_from] = :pubkey_file
+                identity[:pubkey_file] = public_key_file
+              else
+                identity[:load_from] = :privkey_file
+              end
+              identity.merge(privkey_file: file)
             end
           end.compact
         end
 
+        def readable_file?(path)
+          File.file?(path) && File.readable?(path)
+        end
+
         # Prepared identities from user key_data, preserving their order and sources.
         def prepare_identities_from_data
           key_data.map do |data|
@@ -202,18 +212,18 @@ module Net
           end
         end
 
-        # Load prepared identities. Private key decryption errors ignored if passphrase was not prompted.
-        def load_identities(identities, ask_passphrase)
+        # Load prepared identities. Private key decryption errors ignored if ignore_decryption_errors
+        def load_identities(identities, ask_passphrase, ignore_decryption_errors)
           identities.map do |identity|
             begin
               case identity[:load_from]
               when :pubkey_file
-                key = KeyFactory.load_public_key(identity[:file] + ".pub")
-                { :public_key => key, :from => :file, :file => identity[:file] }
+                key = KeyFactory.load_public_key(identity[:pubkey_file])
+                { :public_key => key, :from => :file, :file => identity[:privkey_file] }
               when :privkey_file
-                private_key = KeyFactory.load_private_key(identity[:file], options[:passphrase], ask_passphrase)
+                private_key = KeyFactory.load_private_key(identity[:privkey_file], options[:passphrase], ask_passphrase)
                 key = private_key.send(:public_key)
-                { :public_key => key, :from => :file, :file => identity[:file], :key => private_key }
+                { :public_key => key, :from => :file, :file => identity[:privkey_file], :key => private_key }
               when :data
                 private_key = KeyFactory.load_data_private_key(identity[:data], options[:passphrase], ask_passphrase)
                 key = private_key.send(:public_key)
@@ -222,12 +232,12 @@ module Net
                 identity
               end
 
-            rescue OpenSSL::PKey::RSAError, OpenSSL::PKey::DSAError, OpenSSL::PKey::ECError => e
-              if ask_passphrase
+            rescue OpenSSL::PKey::RSAError, OpenSSL::PKey::DSAError, OpenSSL::PKey::ECError, ArgumentError => e
+              if ignore_decryption_errors
+                identity
+              else
                 process_identity_loading_error(identity, e)
                 nil
-              else
-                identity
               end
             rescue Exception => e
               process_identity_loading_error(identity, e)
@@ -239,9 +249,9 @@ module Net
         def process_identity_loading_error(identity, e)
           case identity[:load_from]
           when :pubkey_file
-            error { "could not load public key file `#{identity[:file]}': #{e.class} (#{e.message})" }
+            error { "could not load public key file `#{identity[:pubkey_file]}': #{e.class} (#{e.message})" }
           when :privkey_file
-            error { "could not load private key file `#{identity[:file]}': #{e.class} (#{e.message})" }
+            error { "could not load private key file `#{identity[:privkey_file]}': #{e.class} (#{e.message})" }
           else
             raise e
           end
