@@ -210,6 +210,9 @@ module Net; module SSH; module Connection
       readers, writers, = Net::SSH::Compat.io_select(r, w, nil, io_select_wait(wait))
 
       postprocess(readers, writers)
+    rescue
+      force_channel_cleanup_on_close if closed?
+      raise
     end
 
     # This is called internally as part of #process. It dispatches any
@@ -220,7 +223,7 @@ module Net; module SSH; module Connection
     def preprocess
       return false if block_given? && !yield(self)
       dispatch_incoming_packets
-      channels.each { |id, channel| channel.process unless channel.closing? }
+      channels.each { |id, channel| channel.process unless channel.local_closed? }
       return false if block_given? && !yield(self)
       return true
     end
@@ -455,6 +458,14 @@ module Net; module SSH; module Connection
       old
     end
 
+    def cleanup_channel(channel)
+        if channel.local_closed? and channel.remote_closed?
+          info { "#{host} delete channel #{channel.local_id} which closed locally and remotely" }
+          channels.delete(channel.local_id)
+      end
+    end
+
+
     private
 
       # Read all pending packets from the connection and dispatch them as
@@ -467,6 +478,9 @@ module Net; module SSH; module Connection
 
           send(MAP[packet.type], packet)
         end
+      rescue
+        force_channel_cleanup_on_close if closed?
+        raise
       end
 
       # Returns the next available channel id to be assigned, and increments
@@ -475,6 +489,16 @@ module Net; module SSH; module Connection
         @channel_id_counter += 1
       end
 
+      def force_channel_cleanup_on_close
+        channels.each do |id, channel|
+          channel.remote_closed!
+          channel.close
+
+          cleanup_channel(channel)
+          channel.do_close
+        end
+      end
+
       # Invoked when a global request is received. The registered global
       # request callback will be invoked, if one exists, and the necessary
       # reply returned.
@@ -583,9 +607,10 @@ module Net; module SSH; module Connection
         info { "channel_close: #{packet[:local_id]}" }
 
         channel = channels[packet[:local_id]]
+        channel.remote_closed!
         channel.close
 
-        channels.delete(packet[:local_id])
+        cleanup_channel(channel)
         channel.do_close
       end
 
@@ -600,9 +625,11 @@ module Net; module SSH; module Connection
       end
 
       def io_select_wait(wait)
-        return wait if wait
-        return wait unless @keepalive.enabled?
-        @keepalive.interval
+        if @keepalive.enabled?
+          [wait, @keepalive.interval].compact.min
+        else
+          wait
+        end
       end
 
       MAP = Constants.constants.inject({}) do |memo, name|
