@@ -1,6 +1,7 @@
 require 'common'
 require 'net/ssh/config'
 require 'pathname'
+require 'tempfile'
 
 class TestConfig < Test::Unit::TestCase
   def test_home_should_be_absolute_path
@@ -41,6 +42,33 @@ class TestConfig < Test::Unit::TestCase
     assert !config.key?('compression')
   end
 
+  def test_load_with_pattern_does_match
+    data = %q{
+      Host test.*
+        Port 1234
+        Compression no
+    }
+    with_config_from_data data do |f|
+      config = Net::SSH::Config.load(f, "test.host")
+      assert_equal 1234, config['port']
+    end
+  end
+
+  def test_load_with_regex_chars
+    data = %q{
+      Host |
+        Port 1234
+        Compression no
+    }
+    with_config_from_data data do |f|
+      config = Net::SSH::Config.load(f, "test.host")
+      assert_equal nil, config['port']
+      config = Net::SSH::Config.load(f, "|")
+      assert_equal 1234, config['port']
+    end
+  end
+
+
   def test_for_should_load_all_files_and_translate_to_net_ssh_options
     config = Net::SSH::Config.for("test.host", [config(:exact_match), config(:wild_cards)])
     assert_equal 1234, config[:port]
@@ -94,7 +122,9 @@ class TestConfig < Test::Unit::TestCase
       'pubkeyauthentication'    => true,
       'rekeylimit'              => 1024,
       'sendenv'                 => "LC_*",
-      'numberofpasswordprompts' => '123'
+      'numberofpasswordprompts' => '123',
+      'serveraliveinterval'     => '2',
+      'serveralivecountmax'     => '4'
     }
 
     net_ssh = Net::SSH::Config.translate(open_ssh)
@@ -113,6 +143,9 @@ class TestConfig < Test::Unit::TestCase
     assert_equal "127.0.0.1", net_ssh[:bind_address]
     assert_equal [/^LC_.*$/], net_ssh[:send_env]
     assert_equal 123,       net_ssh[:number_of_password_prompts]
+    assert_equal 4,         net_ssh[:keepalive_maxcount]
+    assert_equal 2,         net_ssh[:keepalive_interval]
+    assert_equal true,         net_ssh[:keepalive]
   end
 
   def test_translate_should_turn_off_authentication_methods
@@ -224,4 +257,12 @@ class TestConfig < Test::Unit::TestCase
     def config(name)
       "test/configs/#{name}"
     end
+
+    def with_config_from_data(data, &block)
+      Tempfile.open('config') do |f|
+        f.write(data)
+        f.close
+        yield(f.path)
+      end
+    end
 end
